/* READS A G96 (GROMOS 96) FILE AND RETURNS A STRUCTURE CONTAINING        */
/* ITS INFORMATION                                                        */
/* The atom positions are read from the g96 file in nm                    */
/* No processing is done in residue names, atom names, etc, so the        */
/* format is the GROMOS format.                                           */
/* This version reads the boxsize and outputs a flag stating that a box   */
/* was read or not.                                                       */
/*                                                                        */
/*   Claudio Soares, ITQB Oeiras 1999, 2000-11-21, Changed 2001-04-20     */
/* This version is different from the one used in universal               */
/*------------------------------------------------------------------------*/

#include<stdio.h>
#include<string.h>
#include<stdlib.h>

#include "mol_system.h"

void read_g96_file(char *filename,             /* The filename to be read */
		   struct mol_system *aux,     /* The molecular structur  */
		   double *boxsize_x,          /* The size of the box     */
		   double *boxsize_y,
		   double *boxsize_z,
		   int *boxread)               /* 1 if a box was read and */
                                               /* 0 otherwise             */
{

	FILE *ifile;
	char string[256],string_aux[256],string_aux1[256],string_aux2[256];
	char string_aux3[256];
	int i,i_aux,contcycle,endoffile;
	double a1,a2,a3;
	
	/* Templates */
	int fread_until_end_of_line_no_comment(FILE *,char * );
	void copy_substring_to_string(char *,char *,int,int);
	void right_adjust_string(char *);

	/* A box wasn't read yet */
	*boxread=0;

	if ( (ifile=fopen(filename, "r")) == NULL )
	{
		printf("Input file %s can not be open.\n",filename);
		exit(2);
	}
	else
	{
		
		/* Read until record TITLE is found */
	        contcycle=1;
	        while (contcycle==1)
		{
		  fread_until_end_of_line_no_comment(ifile,string);
		  copy_substring_to_string(string,string_aux,1,5);
		  if ( strcmp(string_aux,"TITLE")==0 )
		  {
		    /* Found record TITLE */
		    contcycle=0;
		  }
		}

		/* Read title */
		fread_until_end_of_line_no_comment(ifile,aux->title);
		/* See if the next record is not END itself. In this case
		   change it to a standard string                         */
		if ( strcmp(aux->title,"END")==0 ) {
		  strcpy(aux->title,"GROMOS96 conformation file");
		}

		/* Read until a record POSITION, REFPOSITION or POSRESSPEC 
                   is found                                               */
	        contcycle=1;
	        while (contcycle==1)
		{
		  fread_until_end_of_line_no_comment(ifile,string);
		  copy_substring_to_string(string,string_aux1,1,8);
		  copy_substring_to_string(string,string_aux2,1,11);
		  copy_substring_to_string(string,string_aux3,1,10);
		  if ( (strcmp(string_aux1,"POSITION")==0)||
                       (strcmp(string_aux2,"REFPOSITION")==0)||
		       (strcmp(string_aux3,"POSRESSPEC")==0) )
		  {
		    /* Found record*/
		    contcycle=0;
		  }
		}


		/* Read coordinates until the record END is found */
		aux->no_of_atoms=0;
	        contcycle=1;
	        while (contcycle==1)
		{
		  fread_until_end_of_line_no_comment(ifile,string);
		  copy_substring_to_string(string,string_aux2,1,3);
		  if (strcmp(string_aux2,"END")==0)
		  {
		    /* END record found */
		    contcycle=0;
		  } else {
		    /* Do processing of coordinates */
		  
                    /* Convertion of FORTRAN FORMAT(I5,2(1X,A5),I7,3F15.9) */
		  
                    /* Read residue number */
                    copy_substring_to_string(string,string_aux,1,5);
                    aux->resnumber[aux->no_of_atoms]=atoi(string_aux);
		  	
                    /* Read residue name */
                    copy_substring_to_string(string,string_aux,7,11);
                    strcpy(aux->resname[aux->no_of_atoms],string_aux);
                      
                    /* Read atom name */
                    copy_substring_to_string(string,string_aux,13,17);
                    strcpy(aux->aname[aux->no_of_atoms],string_aux);
                      
		  
                    /* Read atom number */
                    aux->anumber[aux->no_of_atoms]=aux->no_of_atoms+1; 
		    /* Atom number is taken to be the position */
                      
                    /* Read XYZ positions  */
                    copy_substring_to_string(string,string_aux,25,39);
                    aux->conf[aux->no_of_atoms][0]=atof(string_aux);
                    copy_substring_to_string(string,string_aux,40,54);
                    aux->conf[aux->no_of_atoms][1]=atof(string_aux);
                    copy_substring_to_string(string,string_aux,55,69);
                    aux->conf[aux->no_of_atoms][2]=atof(string_aux);
		  
                      
                    /* Place Atom Type */
                    strcpy(aux->atype[aux->no_of_atoms],"ATOM ");
                      
                    /* Place properties to zero */
                    /* Since the first element can be used as occupancy it */
                    /* will be initiallized at 1.0 */
                    aux->prop[aux->no_of_atoms][0]=1.0;
                    aux->prop[aux->no_of_atoms][1]=0.0;
                    aux->prop[aux->no_of_atoms][2]=0.0;
		  
		    (aux->no_of_atoms)++;
		  
		  }
		}

		/* Read until a record BOX is found or until endoffile */
	        contcycle=1;
	        while (contcycle==1)
		{
		  /* Set boxsizes to zero by default */
		  *boxsize_x=0.0;
		  *boxsize_y=0.0;
		  *boxsize_z=0.0;
		  endoffile=fread_until_end_of_line_no_comment(ifile,string);
		  /* Check if endoffile was reached */
		  if (endoffile==1) { 
		    copy_substring_to_string(string,string_aux2,1,3);
		    if (strcmp(string_aux2,"BOX")==0)
		    {
		      /* BOX record found */
		      *boxread=1;
		      fread_until_end_of_line_no_comment(ifile,string);
		      /* Convertion of FORTRAN FORMAT(3F15.9) */
		      copy_substring_to_string(string,string_aux,1,15);
		      *boxsize_x=atof(string_aux);
		      copy_substring_to_string(string,string_aux,16,30);
		      *boxsize_y=atof(string_aux);
		      copy_substring_to_string(string,string_aux,31,45);
		      *boxsize_z=atof(string_aux);
		      contcycle=0;  /* Bail out */
		    } 
		  } else {
		    /* End of file was reached */
		    contcycle=0; /* Bail out */
		  }
		}

		/* Set all strings to null terminated */
		for (i=0;i<aux->no_of_atoms;++i) {
		  /* Place the string termination in the last character of
		     the string. Note that this can destroy part of it if
		     the size is not sufficient to hold all the characters */
		  aux->aname[i][sizeof(aux->aname[i])/sizeof(char)-1]='\0';
		  aux->resname[i][sizeof(aux->resname[i])/sizeof(char)-1]='\0';
		  aux->atype[i][sizeof(aux->atype[i])/sizeof(char)-1]='\0';
		}

		fclose(ifile);
	}
	
	
}


