#include<stdio.h>
#include<string.h>
#include<stdlib.h>

/* Including molecular topology needed constants and data structures */
#include "topology_gromos96.h"


/*******************************************************************
  Reads a GROMOS96 formatted topology file

  This version only reads some fields of the molecular topology file

  Claudio M. Soares, ITQB Oeiras 2000, revision 2002-08-30
 ******************************************************************/
void readg96topology(char *filename,struct g96topology *topology) {

  FILE *ifile;
  char line[256];

  int i;


  /* Templates */
  int fread_until_token(FILE *,char *);
  int fread_until_end_of_line_no_comment(FILE *,char *);
  void read_residue_information(FILE *,struct g96topology *);
  void read_atomic_information(FILE *,struct g96topology *);
  void read_lj_parameters(FILE *,struct g96topology *);
  void read_bond_types(FILE *,struct g96topology *);
  void read_bond_hydrogen(FILE *,struct g96topology *);
  void read_bond_non_hydrogen(FILE *,struct g96topology *);
  void read_bondangle_types(FILE *,struct g96topology *);
  void read_bondangle_hydrogen(FILE *,struct g96topology *);
  void read_bondangle_non_hydrogen(FILE *,struct g96topology *);
  void read_improperdihedral_types(FILE *,struct g96topology *);
  void read_improperdihedral_hydrogen(FILE *,struct g96topology *);
  void read_improperdihedral_non_hydrogen(FILE *,struct g96topology *);
  void read_dihedral_types(FILE *,struct g96topology *);
  void read_dihedral_hydrogen(FILE *,struct g96topology *);
  void read_dihedral_non_hydrogen(FILE *,struct g96topology *);


  if ( (ifile=fopen(filename, "r")) == NULL ) {
    printf("Input file %s can not be open.\n",filename);
    exit(2);
  } else {

    /* Read title */
    if (fread_until_token(ifile,"TITLE")==1) {
      fread_until_end_of_line_no_comment(ifile,topology->title);
    } else {
      printf("Record TITLE not found\n");
      exit(2);
    }
    
    /* Read number of atom types */
    /* Find the record ATOMTYPENAME */
    if (fread_until_token(ifile,"ATOMTYPENAME")==1) {
      /* Read the number of atom types */
      fread_until_end_of_line_no_comment(ifile,line);
      /* Read the number of atom types */
      sscanf(line,"%d",&(topology->nratt));
      /* Read the names of the atom types */
      for (i=0;i<topology->nratt;++i) {
	fread_until_end_of_line_no_comment(ifile,line);
	sscanf(line,"%s",&(topology->atom_type_name[i]));
	/* Place the string termination in the last position of the array */
	/* This will reduce the effective storage space of the string and */
	/* destroy the last character                                     */
	topology->atom_type_name[i][sizeof(topology->atom_type_name[i])/sizeof(char)-1]='\0';
      }
    } else {
      printf("Record ATOMTYPENAME not found\n");
      exit(2);
    }

    /* Read residue information */
    if (fread_until_token(ifile,"RESNAME")==1) {
      /* Read residue information */
      read_residue_information(ifile,topology);
    } else {
      printf("Record RESNAME not found\n");
      exit(2);
    }
    


    /* Read atomic information */
    /* Find the record SOLUTEATOM */
    if (fread_until_token(ifile,"SOLUTEATOM")==1) {
      /* Read atomic information */
      read_atomic_information(ifile,topology);
    } else {
      printf("Record SOLUTEATOM not found\n");
      exit(2);
    }

    /* Read bond informations */
    /* Find the record BONDTYPE */
    if (fread_until_token(ifile,"BONDTYPE")==1) {
      /* Read the number of bondtypes */
      fread_until_end_of_line_no_comment(ifile,line);
      /* Read the number of atom types */
      sscanf(line,"%d",&(topology->nbty));
      /* Read bond types */
      read_bond_types(ifile,topology);
    } else {
      printf("Record BONDTYPE not found\n");
      exit(2);
    }

    /* Read bonds involving at least one H-atom */
    /* Find the record BONDH */
    if (fread_until_token(ifile,"BONDH")==1) {
      /* Read the number of bonds */
      fread_until_end_of_line_no_comment(ifile,line);
      /* Read the number of bonds */
      sscanf(line,"%d",&(topology->nbonh));
      /* Read bond */
      read_bond_hydrogen(ifile,topology);
    } else {
      printf("Record BONDH not found\n");
      exit(2);
    }

    /* Read bonds involving heteroatoms */
    /* Find the record BOND */
    if (fread_until_token(ifile,"BOND")==1) {
      /* Read the number of bonds */
      fread_until_end_of_line_no_comment(ifile,line);
      /* Read the number of bonds */
      sscanf(line,"%d",&(topology->nbon));
      /* Read bond */
      read_bond_non_hydrogen(ifile,topology);
    } else {
      printf("Record BOND not found\n");
      exit(2);
    }

    /* Read bond angle information */
    /* Find the record BONDANGLETYPE */
    if (fread_until_token(ifile,"BONDANGLETYPE")==1) {
      /* Read the number of bond angle types */
      fread_until_end_of_line_no_comment(ifile,line);
      sscanf(line,"%d",&(topology->ntty));
      /* Read bond angle types */
      read_bondangle_types(ifile,topology);
    } else {
      printf("Record BONDANGLETYPE not found\n");
      exit(2);
    }

    /* Read bondangles involving at least one H-atom */
    /* Find the record BONDANGLEH */
    if (fread_until_token(ifile,"BONDANGLEH")==1) {
      /* Read the number of bondangles */
      fread_until_end_of_line_no_comment(ifile,line);
      sscanf(line,"%d",&(topology->ntheh));
      /* Read bondangles */
      read_bondangle_hydrogen(ifile,topology);
    } else {
      printf("Record BONDANGLEH not found\n");
      exit(2);
    }

    /* Read bondangles involving no H-atoms */
    /* Find the record BONDANGLE */
    if (fread_until_token(ifile,"BONDANGLE")==1) {
      /* Read the number of bondangles */
      fread_until_end_of_line_no_comment(ifile,line);
      sscanf(line,"%d",&(topology->nthe));
      /* Read bondangles */
      read_bondangle_non_hydrogen(ifile,topology);
    } else {
      printf("Record BONDANGLE not found\n");
      exit(2);
    }


    /* Read improper dihedral information */
    /* Find the record IMPDIHEDRALTYPE */
    if (fread_until_token(ifile,"IMPDIHEDRALTYPE")==1) {
      /* Read the number of improper dihedral types */
      fread_until_end_of_line_no_comment(ifile,line);
      sscanf(line,"%d",&(topology->nqty));
      /* Read improper diheral types */
      read_improperdihedral_types(ifile,topology);
    } else {
      printf("Record IMPDIHEDRALTYPE not found\n");
      exit(2);
    }

    /* Read improper dihedrals involving at least one H-atom */
    /* Find the record IMPDIHEDRALH*/
    if (fread_until_token(ifile,"IMPDIHEDRALH")==1) {
      /* Read the number of improper dihedrals */
      fread_until_end_of_line_no_comment(ifile,line);
      sscanf(line,"%d",&(topology->nqhih));
      /* Read bondangles */
      read_improperdihedral_hydrogen(ifile,topology);
    } else {
      printf("Record IMPDIHEDRALH not found\n");
      exit(2);
    }

    /* Read improper dihedrals involving no H-atoms */
    /* Find the record IMPDIHEDRAL */
    if (fread_until_token(ifile,"IMPDIHEDRAL")==1) {
      /* Read the number of improper dihedrals */
      fread_until_end_of_line_no_comment(ifile,line);
      sscanf(line,"%d",&(topology->nqhi));
      /* Read improper dihedrals */
      read_improperdihedral_non_hydrogen(ifile,topology);
    } else {
      printf("Record IMPDIHEDRAL not found\n");
      exit(2);
    }

    /* Read dihedral information */
    /* Find the record DIHEDRALTYPE */
    if (fread_until_token(ifile,"DIHEDRALTYPE")==1) {
      /* Read the number of dihedral types */
      fread_until_end_of_line_no_comment(ifile,line);
      sscanf(line,"%d",&(topology->npty));
      /* Read diheral types */
      read_dihedral_types(ifile,topology);
    } else {
      printf("Record DIHEDRALTYPE not found\n");
      exit(2);
    }


    /* Read dihedrals involving at least one H-atom */
    /* Find the record DIHEDRALH*/
    if (fread_until_token(ifile,"DIHEDRALH")==1) {
      /* Read the number of dihedrals */
      fread_until_end_of_line_no_comment(ifile,line);
      sscanf(line,"%d",&(topology->nphih));
      /* Read dihedrals */
      read_dihedral_hydrogen(ifile,topology);
    } else {
      printf("Record DIHEDRALH not found\n");
      exit(2);
    }

    /* Read dihedrals involving no H-atoms */
    /* Find the record DIHEDRAL */
    if (fread_until_token(ifile,"DIHEDRAL")==1) {
      /* Read the number of dihedrals */
      fread_until_end_of_line_no_comment(ifile,line);
      sscanf(line,"%d",&(topology->nphi));
      /* Read dihedrals */
      read_dihedral_non_hydrogen(ifile,topology);
    } else {
      printf("Record DIHEDRAL not found\n");
      exit(2);
    }









    /* Read Lennard-Jones interaction parameters */
    /* Find the record LJPARAMETERS */
    if (fread_until_token(ifile,"LJPARAMETERS")==1) {
      /* Read LJ matrix parameters */
      read_lj_parameters(ifile,topology);
    } else {
      printf("Record LJPARAMETERS not found\n");
      exit(2);
    }


    fclose(ifile);
  }


} /* readg96topology */






/*************************************************************************/
/*  Read the lines with atomic information of the molecular topology     */
/*  file                                                                 */
/* Claudio M. Soares, ITQB Oeiras, 2000                                  */
/*************************************************************************/
void read_residue_information(FILE *ifile,struct g96topology *topology) {


  char line[256];
  int i;

  /* Templates */
  int fread_until_end_of_line_no_comment(FILE *,char *);
  void left_adjust_string(char *);

  /* Read number of residues */
  fread_until_end_of_line_no_comment(ifile,line);
  sscanf(line,"%d",&(topology->no_of_residues));

  /* Read the residue names */
  for (i=0;i<topology->no_of_residues;++i) {
    fread_until_end_of_line_no_comment(ifile,line);
    sscanf(line,"%s",&(topology->residues[i]));
    /* Place the string termination in the last position of the array */
    /* This will reduce the effective storage space of the string and */
    /* destroy the last character                                     */
    topology->residues[i][sizeof(topology->residues[i])/sizeof(char)-1]='\0';
    left_adjust_string(topology->residues[i]);
  }

} /* read_residue_information */



void read_atomic_information(FILE *ifile,struct g96topology *topology) {


  char line[256];
  int i;
  

  /* Templates */
  int fread_until_end_of_line_no_comment(FILE *,char *);
  void left_adjust_string(char *);

  fread_until_end_of_line_no_comment(ifile,line);
  /* Read the number of atoms */
  sscanf(line,"%d",&(topology->no_of_atoms));
  /* For each atom read several things */
  for (i=0;i<topology->no_of_atoms;++i) {

    /* Read first line */
    fread_until_end_of_line_no_comment(ifile,line);

    /* There are six possible exclusions in the first line */
    sscanf(line,"%d %d %s %d %lf %lf %d %d %d %d %d %d %d %d",
           &(topology->atom_number[i]),
	   &(topology->residue_number[i]),
	   &(topology->atom_name[i]),
	   &(topology->atom_iac[i]),
	   &(topology->atom_mass[i]),
	   &(topology->atom_charge[i]),
	   &(topology->atom_charge_group_code[i]),
	   &(topology->atom_ine[i]),
	   &(topology->atom_excl[i][0]),
	   &(topology->atom_excl[i][1]),
	   &(topology->atom_excl[i][2]),
	   &(topology->atom_excl[i][3]),
	   &(topology->atom_excl[i][4]),
	   &(topology->atom_excl[i][5]) );

    /* Place the string termination in the last position of the array */
    /* This will reduce the effective storage space of the string and */
    /* destroy the last character                                     */
    topology->atom_name[i][sizeof(topology->atom_name[i])/sizeof(char)-1]='\0';

    /* Left adjust strings */
    left_adjust_string( topology->atom_name[i] );


    /* If there are more exclusions than the ones fitting one line */
    if (topology->atom_ine[i]>6) {
      /* Read more 16 new exclusions */
      fread_until_end_of_line_no_comment(ifile,line);
      sscanf(line,"%d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d",
	     &(topology->atom_excl[i][6]),
	     &(topology->atom_excl[i][7]),
	     &(topology->atom_excl[i][8]),
	     &(topology->atom_excl[i][9]),
	     &(topology->atom_excl[i][10]),
	     &(topology->atom_excl[i][11]),
	     &(topology->atom_excl[i][12]),
	     &(topology->atom_excl[i][13]),
	     &(topology->atom_excl[i][14]),
	     &(topology->atom_excl[i][15]),
	     &(topology->atom_excl[i][16]),
	     &(topology->atom_excl[i][17]),
	     &(topology->atom_excl[i][18]),
	     &(topology->atom_excl[i][19]),
	     &(topology->atom_excl[i][20]),
	     &(topology->atom_excl[i][21]) );
      if (topology->atom_ine[i]>22) {
	/* Read more 16 new exclusions */
	fread_until_end_of_line_no_comment(ifile,line);
	sscanf(line,"%d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d",
	       &(topology->atom_excl[i][22]),
	       &(topology->atom_excl[i][23]),
	       &(topology->atom_excl[i][24]),
	       &(topology->atom_excl[i][25]),
	       &(topology->atom_excl[i][26]),
	       &(topology->atom_excl[i][27]),
	       &(topology->atom_excl[i][28]),
	       &(topology->atom_excl[i][29]),
	       &(topology->atom_excl[i][30]),
	       &(topology->atom_excl[i][31]),
	       &(topology->atom_excl[i][32]),
	       &(topology->atom_excl[i][33]),
	       &(topology->atom_excl[i][34]),
	       &(topology->atom_excl[i][35]),
	       &(topology->atom_excl[i][36]),
	       &(topology->atom_excl[i][37]) );
	if (topology->atom_ine[i]>38) {
	  /* Read more 16 new exclusions */
	  fread_until_end_of_line_no_comment(ifile,line);
	  sscanf(line,"%d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d",
	         &(topology->atom_excl[i][38]),
	         &(topology->atom_excl[i][39]),
	         &(topology->atom_excl[i][40]),
	         &(topology->atom_excl[i][41]),
	         &(topology->atom_excl[i][42]),
	         &(topology->atom_excl[i][43]),
	         &(topology->atom_excl[i][44]),
	         &(topology->atom_excl[i][45]),
	         &(topology->atom_excl[i][46]),
	         &(topology->atom_excl[i][47]),
	         &(topology->atom_excl[i][48]),
	         &(topology->atom_excl[i][49]),
	         &(topology->atom_excl[i][50]),
	         &(topology->atom_excl[i][51]),
	         &(topology->atom_excl[i][52]),
	         &(topology->atom_excl[i][53]) );
	}
      }
    }

    /* Read second line containing the 1-4 exclusions */
    /* Six exclusions can  be present in this file */
    fread_until_end_of_line_no_comment(ifile,line);
    sscanf(line,"%d %d %d %d %d %d %d",
	   &(topology->atom_ine14[i]),
	   &(topology->atom_excl_1_4[i][0]),
	   &(topology->atom_excl_1_4[i][1]),
	   &(topology->atom_excl_1_4[i][2]),
	   &(topology->atom_excl_1_4[i][3]),
	   &(topology->atom_excl_1_4[i][4]),
	   &(topology->atom_excl_1_4[i][5]) );

    /* If there are more exclusions than the ones fitting one line */
    if (topology->atom_ine14[i]>6) {
      /* Read 16 new exclusions */
      fread_until_end_of_line_no_comment(ifile,line);
      sscanf(line,"%d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d",
	     &(topology->atom_excl_1_4[i][6]),
	     &(topology->atom_excl_1_4[i][7]),
	     &(topology->atom_excl_1_4[i][8]),
	     &(topology->atom_excl_1_4[i][9]),
	     &(topology->atom_excl_1_4[i][10]),
	     &(topology->atom_excl_1_4[i][11]),
	     &(topology->atom_excl_1_4[i][12]),
	     &(topology->atom_excl_1_4[i][13]),
	     &(topology->atom_excl_1_4[i][14]),
	     &(topology->atom_excl_1_4[i][15]),
	     &(topology->atom_excl_1_4[i][16]),
	     &(topology->atom_excl_1_4[i][17]),
	     &(topology->atom_excl_1_4[i][18]),
	     &(topology->atom_excl_1_4[i][19]),
	     &(topology->atom_excl_1_4[i][20]),
	     &(topology->atom_excl_1_4[i][21]) );

      if (topology->atom_ine14[i]>22) {
	/* Read more 16 new exclusions */
	fread_until_end_of_line_no_comment(ifile,line);
	sscanf(line,"%d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d",
	       &(topology->atom_excl_1_4[i][22]),
	       &(topology->atom_excl_1_4[i][23]),
	       &(topology->atom_excl_1_4[i][24]),
	       &(topology->atom_excl_1_4[i][25]),
	       &(topology->atom_excl_1_4[i][26]),
	       &(topology->atom_excl_1_4[i][27]),
	       &(topology->atom_excl_1_4[i][28]),
	       &(topology->atom_excl_1_4[i][29]),
	       &(topology->atom_excl_1_4[i][30]),
	       &(topology->atom_excl_1_4[i][31]),
	       &(topology->atom_excl_1_4[i][32]),
	       &(topology->atom_excl_1_4[i][33]),
	       &(topology->atom_excl_1_4[i][34]),
	       &(topology->atom_excl_1_4[i][35]),
	       &(topology->atom_excl_1_4[i][36]),
	       &(topology->atom_excl_1_4[i][37]) );

	if (topology->atom_ine14[i]>38) {
	  /* Read more 16 new exclusions */
	  fread_until_end_of_line_no_comment(ifile,line);
	  sscanf(line,"%d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d",
	         &(topology->atom_excl_1_4[i][38]),
	         &(topology->atom_excl_1_4[i][39]),
	         &(topology->atom_excl_1_4[i][40]),
	         &(topology->atom_excl_1_4[i][41]),
	         &(topology->atom_excl_1_4[i][42]),
	         &(topology->atom_excl_1_4[i][43]),
	         &(topology->atom_excl_1_4[i][44]),
	         &(topology->atom_excl_1_4[i][45]),
	         &(topology->atom_excl_1_4[i][46]),
	         &(topology->atom_excl_1_4[i][47]),
	         &(topology->atom_excl_1_4[i][48]),
	         &(topology->atom_excl_1_4[i][49]),
	         &(topology->atom_excl_1_4[i][50]),
	         &(topology->atom_excl_1_4[i][51]),
	         &(topology->atom_excl_1_4[i][52]),
	         &(topology->atom_excl_1_4[i][53]) );
	}
      }
    }


  }


  /* Assign residue names to each atom */
  for (i=0;i<topology->no_of_atoms;++i) {
    strcpy(topology->residue_name[i],
	   topology->residues[topology->residue_number[i]-1]);
  }

} /* read_atomic_information */




/*************************************************************************/
/*  Read the lines with Lennard-Jones parameters from of the molecular   */
/*  topology file                                                        */
/* Claudio M. Soares, ITQB Oeiras, 2000                                  */
/*************************************************************************/
void read_lj_parameters(FILE *ifile,struct g96topology *topology) {

  char line[256];
  int i,j;
  

  /* Templates */
  int fread_until_end_of_line_no_comment(FILE *,char *);


  /* Read the first line with NRATT*(NRATT+1)/2 */
  fread_until_end_of_line_no_comment(ifile,line);

  for (j=0;j<topology->nratt;++j) {
    for (i=0;i<=j;++i) {

      fread_until_end_of_line_no_comment(ifile,line);
      /* Read IAC indices and Lennard-Jones parameters */
      sscanf(line,"%d %d %lE %lE %lE %lE",&(topology->lj_indices[i][j][0]),
	     &(topology->lj_indices[i][j][1]),
	     &(topology->c12[i][j]),
	     &(topology->c6[i][j]),
	     &(topology->cs12[i][j]),
	     &(topology->cs6[i][j]) );

    }
  }

  
  /* Symmetrize the array (only the upper half matrix was read) */
  for (i=0;i<topology->nratt-1;++i) {
    for (j=i+1;j<topology->nratt;++j) {
      topology->lj_indices[j][i][0]=j+1;
      topology->lj_indices[j][i][1]=i+1;
      topology->c12[j][i]=topology->c12[i][j];
      topology->c6[j][i]=topology->c6[i][j];
      topology->cs12[j][i]=topology->cs12[i][j];
      topology->cs6[j][i]=topology->cs6[i][j];
    }
  }


} /* read_lj_parameters */



/*************************************************************************/
/*  Read the lines with bond type parameters from of the molecular       */
/*  topology file                                                        */
/* Claudio M. Soares, ITQB Oeiras, 2001                                  */
/*************************************************************************/
void read_bond_types(FILE *ifile,struct g96topology *topology) {

  char line[256];
  int i;

  for (i=0;i<topology->nbty;++i) {

    fread_until_end_of_line_no_comment(ifile,line);
    sscanf(line,"%lf %lf",&(topology->cb[i]),&(topology->b0[i]));

  }

} /* read_bond_types */


/*************************************************************************/
/*  Read the lines with bonds involving at least one H-atom              */
/*  Claudio M. Soares, ITQB Oeiras, 2001                                 */
/*************************************************************************/
void read_bond_hydrogen(FILE *ifile,struct g96topology *topology) {

  char line[256];
  int i;

  for (i=0;i<topology->nbonh;++i) {
    fread_until_end_of_line_no_comment(ifile,line);
    sscanf(line,"%d %d %d",&(topology->ibh[i]),
	                   &(topology->jbh[i]),
	                   &(topology->icbh[i]));
  }

} /* read_bond_hydrogen */


/*************************************************************************/
/*  Read the lines with bonds involving non-H-atoms                      */
/*  Claudio M. Soares, ITQB Oeiras, 2001                                 */
/*************************************************************************/
void read_bond_non_hydrogen(FILE *ifile,struct g96topology *topology) {

  char line[256];
  int i;

  for (i=0;i<topology->nbon;++i) {
    fread_until_end_of_line_no_comment(ifile,line);
    sscanf(line,"%d %d %d",&(topology->ib[i]),
	                   &(topology->jb[i]),
	                   &(topology->icb[i]));
  }

} /* read_bond_non_hydrogen */


/*************************************************************************/
/*  Read the lines with bondangle type parameters from of the molecular   */
/*  topology file                                                        */
/* Claudio M. Soares, ITQB Oeiras, 2001                                  */
/*************************************************************************/
void read_bondangle_types(FILE *ifile,struct g96topology *topology) {

  char line[256];
  int i;

  for (i=0;i<topology->ntty;++i) {

    fread_until_end_of_line_no_comment(ifile,line);
    sscanf(line,"%lf %lf",&(topology->ct[i]),&(topology->t0[i]));

  }

} /* read_bondangle_types */


/*************************************************************************/
/*  Read the lines with bondangles involving at least one H-atom         */
/*  Claudio M. Soares, ITQB Oeiras, 2001                                 */
/*************************************************************************/
void read_bondangle_hydrogen(FILE *ifile,struct g96topology *topology) {

  char line[256];
  int i;

  for (i=0;i<topology->ntheh;++i) {
    fread_until_end_of_line_no_comment(ifile,line);
    sscanf(line,"%d %d %d %d",&(topology->ith[i]),
	                      &(topology->jth[i]),
	                      &(topology->kth[i]),
	                      &(topology->icth[i]));
  }

} /* read_bondangle_hydrogen */


/*************************************************************************/
/*  Read the lines with bondangles involving non-H-atoms                 */
/*  Claudio M. Soares, ITQB Oeiras, 2001                                 */
/*************************************************************************/
void read_bondangle_non_hydrogen(FILE *ifile,struct g96topology *topology) {

  char line[256];
  int i;

  for (i=0;i<topology->nthe;++i) {
    fread_until_end_of_line_no_comment(ifile,line);
    sscanf(line,"%d %d %d %d",&(topology->it[i]),
	                      &(topology->jt[i]),
	                      &(topology->kt[i]),	   
	                      &(topology->ict[i]));
  }

} /* read_bondangle_non_hydrogen */


/*************************************************************************/
/*  Read the lines with improper dihedral type parameters                */
/* Claudio M. Soares, ITQB Oeiras, 2001                                  */
/*************************************************************************/
void read_improperdihedral_types(FILE *ifile,struct g96topology *topology) {

  char line[256];
  int i;

  for (i=0;i<topology->nqty;++i) {

    fread_until_end_of_line_no_comment(ifile,line);
    sscanf(line,"%lf %lf",&(topology->cq[i]),&(topology->q0[i]));

  }

} /* read_improperdihedral_types */


/*************************************************************************/
/*  Read the lines with improper dihedrals involving at least one H-atom */
/*  Claudio M. Soares, ITQB Oeiras, 2001                                 */
/*************************************************************************/
void read_improperdihedral_hydrogen(FILE *ifile,struct g96topology *topology) {

  char line[256];
  int i;

  for (i=0;i<topology->nqhih;++i) {
    fread_until_end_of_line_no_comment(ifile,line);
    sscanf(line,"%d %d %d %d %d",&(topology->iqh[i]),
	                         &(topology->jqh[i]),
	                         &(topology->kqh[i]),
	                         &(topology->lqh[i]),
	                         &(topology->icqh[i]));
  }

} /* read_improperdihedral_hydrogen */


/*************************************************************************/
/*  Read the lines with improper dihedrals involving non-H-atoms         */
/*  Claudio M. Soares, ITQB Oeiras, 2001                                 */
/*************************************************************************/
void read_improperdihedral_non_hydrogen(FILE *ifile,
					struct g96topology *topology) {

  char line[256];
  int i;

  for (i=0;i<topology->nqhi;++i) {
    fread_until_end_of_line_no_comment(ifile,line);
    sscanf(line,"%d %d %d %d %d",&(topology->iq[i]),
	                         &(topology->jq[i]),
	                         &(topology->kq[i]),	   
	                         &(topology->lq[i]),	   
	                         &(topology->icq[i]));
  }

} /* read_improperdihedral_non_hydrogen */



/*************************************************************************/
/*  Read the lines with dihedral type parameters                         */
/* Claudio M. Soares, ITQB Oeiras, 2001                                  */
/*************************************************************************/
void read_dihedral_types(FILE *ifile,struct g96topology *topology) {

  char line[256];
  int i;

  for (i=0;i<topology->npty;++i) {

    fread_until_end_of_line_no_comment(ifile,line);
    sscanf(line,"%lf %lf %d",&(topology->cp[i]),
	                     &(topology->pd[i]),
	                     &(topology->np[i]) );
  }

} /* read_dihedral_types */


/*************************************************************************/
/*  Read the lines with dihedrals involving at least one H-atom          */
/*  Claudio M. Soares, ITQB Oeiras, 2001                                 */
/*************************************************************************/
void read_dihedral_hydrogen(FILE *ifile,struct g96topology *topology) {

  char line[256];
  int i;

  for (i=0;i<topology->nphih;++i) {
    fread_until_end_of_line_no_comment(ifile,line);
    sscanf(line,"%d %d %d %d %d",&(topology->iph[i]),
	                         &(topology->jph[i]),
	                         &(topology->kph[i]),
	                         &(topology->lph[i]),
	                         &(topology->icph[i]));
  }

} /* read_dihedral_hydrogen */


/*************************************************************************/
/*  Read the lines with dihedrals involving non-H-atoms                  */
/*  Claudio M. Soares, ITQB Oeiras, 2001                                 */
/*************************************************************************/
void read_dihedral_non_hydrogen(FILE *ifile,
				struct g96topology *topology) {

  char line[256];
  int i;

  for (i=0;i<topology->nphi;++i) {
    fread_until_end_of_line_no_comment(ifile,line);
    sscanf(line,"%d %d %d %d %d",&(topology->ip[i]),
	                         &(topology->jp[i]),
	                         &(topology->kp[i]),	   
	                         &(topology->lp[i]),	   
	                         &(topology->icp[i]));
  }

} /* read_dihedral_non_hydrogen */
