#include<stdio.h>
#include<string.h>
#include<stdlib.h>
#include<math.h>

/* Including molecular topology needed constants and data structures */
#include "topology_gromos96.h"


/*******************************************************************
  Writes a GROMACS topology file (for version 3.X) from a GROMOS96
  topology structure


  Claudio M. Soares, ITQB-UNL, Oeiras, 2002-08-26
 ******************************************************************/
void write_gromacs_topology(char *filename,
			    struct g96topology *topology,
			    int no_solvent_molecules) {

  FILE *ofile;
  char line[256];

  /* Templates */
  void write_atomtypes(FILE *,struct g96topology *);
  void write_nonbond_params(FILE *,struct g96topology *);
  void write_pairtypes(FILE *,struct g96topology *);
  void write_bondstretching_parameters(FILE *,struct g96topology *);
  void write_bondangle_parameters(FILE *,struct g96topology *);
  void write_improper_parameters(FILE *,struct g96topology *);
  void write_dihedral_parameters(FILE *,struct g96topology *);
  void write_atoms(FILE *,struct g96topology *);
  void write_bonds(FILE *,struct g96topology *);
  void write_1_4_interactions(FILE *,struct g96topology *);
  void write_bondangles(FILE *,struct g96topology *);
  void write_dihedrals(FILE *,struct g96topology *);
  void write_impropers(FILE *,struct g96topology *);


  if ( (ofile=fopen(filename, "w")) == NULL ) {
    printf("Output file %s can not be open.\n",filename);
    exit(2);
  } else {

    /* Write header */
    fprintf(ofile,";\n");
    fprintf(ofile,"; This file was generated by convertion from a GROMOS96\n");
    fprintf(ofile,"; topology file\n;\n");
    fprintf(ofile,"; Note that this file does not include any force field\n");
    fprintf(ofile,"; specific file. Instead the definitions of the force\n");
    fprintf(ofile,"; field are included at the top.\n;\n");
    fprintf(ofile,"; Additionally, the define statements are different \n");
    fprintf(ofile,"; from the usual ones used in GROMACS. They all start\n");
    fprintf(ofile,"; with GROMOS96_.\n;\n");
    fprintf(ofile,"; %s\n;\n",topology->title);


    /* Write general things */
    fprintf(ofile,"#define _FF_GROMOS96\n");
    fprintf(ofile,"#define _FF_GROMOS96A\n\n");
    fprintf(ofile,"[ defaults ]\n");
    fprintf(ofile,"; nbfunc	comb-rule	gen-pairs	fudgeLJ	fudgeQQ\n");
    fprintf(ofile,"  1		1		no		1.0	1.0\n\n");

    /* Write atomtypes */
    write_atomtypes(ofile,topology);

    /* Write non-bonded parameters */
    write_nonbond_params(ofile,topology);

    /* Write non-bonded parameters (1-4 interactions) */
    write_pairtypes(ofile,topology);

    /* Write bond-stretching parameters */
    write_bondstretching_parameters(ofile,topology);

    /* Write bond angle parameters */
    write_bondangle_parameters(ofile,topology);

    /* Write improper dihedral parameters */
    write_improper_parameters(ofile,topology);

    /* Write dihedral parameters */
    write_dihedral_parameters(ofile,topology);


    /* Start writting molecular definitions */
    fprintf(ofile,"; Now start the molecular definitions\n;\n;\n\n");
    fprintf(ofile,"[ moleculetype ]\n");
    fprintf(ofile,"; Name            nrexcl\n");
    /* By default the system is named Protein */
    fprintf(ofile,"Protein             3\n\n");
    
    /* Write atom definitions */
    write_atoms(ofile,topology);

    /* Write bonds */
    write_bonds(ofile,topology);
    
    write_1_4_interactions(ofile,topology);

    /* Write bond angles */
    write_bondangles(ofile,topology);

    /* Write dihedrals */
    write_dihedrals(ofile,topology);

    /* Write impropers */
    write_impropers(ofile,topology);

    /* Write settings related with position restraints */
    fprintf(ofile,"; Include Position restraint file\n");
    fprintf(ofile,"#ifdef POSRES\n");
    fprintf(ofile,"#include \"system_posre.itp\"\n");
    fprintf(ofile,"#endif\n\n");
    printf("\nThe position restraints file has been named \"system_posre.itp\".\n");
    printf("Change topology file if you want another name.\n\n");

    /* Write settings related with water */
    fprintf(ofile,"; Include water topology\n");
    fprintf(ofile,"#ifdef FLEX_SPC\n");
    fprintf(ofile,"#include \"flexspc.itp\"\n");
    fprintf(ofile,"#else\n");
    fprintf(ofile,"#include \"spc.itp\"\n");
    fprintf(ofile,"#endif\n\n");
    fprintf(ofile,"#ifdef POSRES_WATER\n");
    fprintf(ofile,"; Position restraint for each water oxygen\n");
    fprintf(ofile,"[ position_restraints ]\n");
    fprintf(ofile,";  i funct       fcx        fcy        fcz\n");
    fprintf(ofile,"   1    1       1000       1000       1000\n");
    fprintf(ofile,"#endif\n\n");

    /* Write settings related with the system */
    fprintf(ofile,"[ system ]\n");
    fprintf(ofile,"; Name\n");
    fprintf(ofile,"%s\n\n",topology->title);

    /* Write settings related with the molecules */
    fprintf(ofile,"[ molecules ]\n");
    fprintf(ofile,"; Compound        #mols\n");
    fprintf(ofile,"Protein             1\n");
    if (no_solvent_molecules>0) {
      fprintf(ofile,"SOL            %6d\n",no_solvent_molecules);
    }
    fprintf(ofile,"\n");

    fclose(ofile);
  }


} /* write_gromacs_topology */



void write_atomtypes(FILE *ofile,struct g96topology *topology) {

  int i;

  void right_adjust_string(char *);

  fprintf(ofile,"[ atomtypes ]\n");
  fprintf(ofile,";name        mass      charge   ptype             c6            c12\n");
  for (i=0;i<topology->nratt;++i) {
    right_adjust_string(
	     topology->atom_type_name[topology->lj_indices[i][i][0]-1]);
    fprintf(ofile,"%5s       0.000       0.000       A  %9.7e  %9.7e\n",
	    topology->atom_type_name[topology->lj_indices[i][i][0]-1],
	    topology->c6[i][i],
	    topology->c12[i][i]);
  }
  fprintf(ofile,"\n");

} /* write_atomtypes */


void write_nonbond_params(FILE *ofile,struct g96topology *topology) {

  int i,j;
  void right_adjust_string(char *);

  /* Right adjust atom type names */
  for (i=0;i<topology->nratt;++i) {
    right_adjust_string(topology->atom_type_name[i]);
  }

  fprintf(ofile,"[ nonbond_params ]\n");
  fprintf(ofile,";      i       j func             c6            c12\n");
  for (i=0;i<topology->nratt;++i) {
    for (j=i;j<topology->nratt;++j) {
      fprintf(ofile,"   %5s   %5s    1  %9.7e  %9.7e\n",
	      topology->atom_type_name[topology->lj_indices[i][j][0]-1],
	      topology->atom_type_name[topology->lj_indices[i][j][1]-1],
	      topology->c6[i][j],
	      topology->c12[i][j]);
    }
  }

  fprintf(ofile,"\n");

} /* write_nonbond_params */



void write_pairtypes(FILE *ofile,struct g96topology *topology) {

  int i,j;
  void right_adjust_string(char *);

  /* Right adjust atom type names */
  for (i=0;i<topology->nratt;++i) {
    right_adjust_string(topology->atom_type_name[i]);
  }

  fprintf(ofile,"[ pairtypes ]\n");
  fprintf(ofile,";      i       j func             c6            c12  ");
  fprintf(ofile,"These are 1-4 interactions\n");
  for (i=0;i<topology->nratt;++i) {
    for (j=i;j<topology->nratt;++j) {
      fprintf(ofile,"   %5s   %5s    1  %9.7e  %9.7e\n",
	      topology->atom_type_name[topology->lj_indices[i][j][0]-1],
	      topology->atom_type_name[topology->lj_indices[i][j][1]-1],
	      topology->cs6[i][j],
	      topology->cs12[i][j]);
    }
  }

  fprintf(ofile,"\n");

} /* write_pairtypes */


void write_bondstretching_parameters(FILE *ofile,
				     struct g96topology *topology) {

  int i;

  fprintf(ofile,";  These are the bond-stretching parameters\n");
  fprintf(ofile,";        define_type         B0          CB\n");
  for (i=0;i<topology->nbty;++i) {
    fprintf(ofile,"#define GROMOS96b_%d     %6.4f  %6.4e\n",
	    i+1,
	    topology->b0[i],
	    topology->cb[i]);
  }
  
  fprintf(ofile,"\n");

} /* write_bondstretching_parameters */



void write_bondangle_parameters(FILE *ofile,struct g96topology *topology) {

  int i;

  fprintf(ofile,";  These are the bond-angle parameters\n");
  fprintf(ofile,";        define_type           T0        CT\n");
  for (i=0;i<topology->ntty;++i) {
    fprintf(ofile,"#define GROMOS96a_%d     %8.2f  %8.2f\n",
	    i+1,
	    topology->t0[i],
	    topology->ct[i]);
  }
  
  fprintf(ofile,"\n");

} /* write_bondangle_parameters */


void write_improper_parameters(FILE *ofile,struct g96topology *topology) {

  int i;

  fprintf(ofile,";  These are the improper dihedral parameters\n");
  fprintf(ofile,";        define_type           Q0        CQ\n");
  for (i=0;i<topology->nqty;++i) {
    /* Note that the force constant must be converted from */
    /* kJ mol-1 degrees-2 to kJ mol-1 rad-2                */
    fprintf(ofile,"#define GROMOS96i_%d     %10.5f  %10.5f\n",
	    i+1,
	    topology->q0[i],
	    topology->cq[i]/( (3.141592654/180.0)*(3.141592654/180.0) ) );
  }
  
  fprintf(ofile,"\n");

} /* write_improper_parameters */


void write_dihedral_parameters(FILE *ofile,struct g96topology *topology) {

  int i;

  fprintf(ofile,";  These are the dihedral parameters\n");
  fprintf(ofile,";        define_type  PD (angle)      CP    NP\n");
  for (i=0;i<topology->npty;++i) {
    /* Note that while GROMOS expects the cosine of the phase shift */
    /* GROMACS expects the phase shift itself                       */
    fprintf(ofile,"#define GROMOS96d_%d     %7.3f  %6.2f     %d\n",
	    i+1,
	    acos(topology->pd[i])*180.0/3.141592654,
	    topology->cp[i],
	    topology->np[i]);
  }
  
  fprintf(ofile,"\n");

} /* write_dihedral_parameters */



void write_atoms(FILE *ofile,struct g96topology *topology) {

  int i,count;
  int atom_charge_group_number[MAXATOMSTOPOLOGY];

  void right_adjust_string(char *);

  /* Assign charge group numbers needed by GROMACS */
  count=1;
  for (i=0;i<topology->no_of_atoms;++i) {
    atom_charge_group_number[i]=count;
    if (topology->atom_charge_group_code[i]==1) {
      ++count;
    }
  }

  fprintf(ofile,"[ atoms ]\n");
  fprintf(ofile,";   nr       type  resnr residue  atom   cgnr     charge       mass  typeB    chargeB      massB\n");
  for (i=0;i<topology->no_of_atoms;++i) {
    right_adjust_string(topology->residue_name[i]);
    right_adjust_string(topology->atom_name[i]);
    fprintf(ofile,"%6d      %5s %6d   %5s %5s %6d   %8.5f  %9.5f\n",
	    i+1,
	    topology->atom_type_name[topology->atom_iac[i]-1],
	    topology->residue_number[i],
	    topology->residue_name[i],
	    topology->atom_name[i],
	    atom_charge_group_number[i],
	    topology->atom_charge[i],
	    topology->atom_mass[i]);
  }
  
  fprintf(ofile,"\n");

} /* write_atoms */


void write_bonds(FILE *ofile,struct g96topology *topology) {

  int i;

  fprintf(ofile,"[ bonds ]\n");
  fprintf(ofile,";   ai     aj  funct            c0            c1            c2            c3\n");

  /* Write out bonds not involving hydrogens */
  for (i=0;i<topology->nbon;++i) {
    fprintf(ofile,"%6d %6d %6d  GROMOS96b_%d\n",
	    topology->ib[i],
	    topology->jb[i],
	    2,                               /* Allways 2 with GROMOS96 */
	    topology->icb[i]);
  }

  /* Write out bonds involving hydrogens */
  for (i=0;i<topology->nbonh;++i) {
    fprintf(ofile,"%6d %6d %6d  GROMOS96b_%d\n",
	    topology->ibh[i],
	    topology->jbh[i],
	    2,                               /* Allways 2 with GROMOS96 */
	    topology->icbh[i]);
  }

  
  fprintf(ofile,"\n");

} /* write_bonds */



void write_bondangles(FILE *ofile,struct g96topology *topology) {

  int i;

  fprintf(ofile,"[ angles ]\n");
  fprintf(ofile,";   ai     aj     ak  funct            c0            c1            c2            c3\n");

  /* Write out bond angles not involving hydrogens */
  for (i=0;i<topology->nthe;++i) {
    fprintf(ofile,"%6d %6d %6d %6d  GROMOS96a_%d\n",
	    topology->it[i],
	    topology->jt[i],
	    topology->kt[i],
	    2,                               /* Allways 2 with GROMOS96 */
	    topology->ict[i]);
  }

  /* Write out bond angles involving hydrogens */
  for (i=0;i<topology->ntheh;++i) {
    fprintf(ofile,"%6d %6d %6d %6d  GROMOS96a_%d\n",
	    topology->ith[i],
	    topology->jth[i],
	    topology->kth[i],
	    2,                               /* Allways 2 with GROMOS96 */
	    topology->icth[i]);
  }

  
  fprintf(ofile,"\n");

} /* write_bondangles */


void write_1_4_interactions(FILE *ofile,struct g96topology *topology) {

  int i,j;

  fprintf(ofile,"[ pairs ]\n");
  fprintf(ofile,";   ai     aj  funct            c0            c1            c2            c3\n");

  for (i=0;i<topology->no_of_atoms;++i) {
    for (j=0;j<topology->atom_ine14[i];++j) {
      fprintf(ofile,"%6d %6d %6d\n",
	      topology->atom_number[i],
	      topology->atom_excl_1_4[i][j],
	      1);                       /* Always 1 in GROMOS96 */
    }
  }


  fprintf(ofile,"\n");

} /* write_1_4_interactions */



void write_dihedrals(FILE *ofile,struct g96topology *topology) {

  int i;

  fprintf(ofile,"[ dihedrals ]\n");
  fprintf(ofile,";   ai     aj     ak     al  funct            c0            c1            c2            c3            c4            c5\n");

  /* Write out dihedral angles not involving hydrogens */
  for (i=0;i<topology->nphi;++i) {
    fprintf(ofile,"%6d %6d %6d %6d %6d  GROMOS96d_%d\n",
	    topology->ip[i],
	    topology->jp[i],
	    topology->kp[i],
	    topology->lp[i],
	    1,                               /* 1 for dihedrals  */
	    topology->icp[i]);
  }

  /* Write out bond angles involving hydrogens */
  for (i=0;i<topology->nphih;++i) {
    fprintf(ofile,"%6d %6d %6d %6d %6d  GROMOS96d_%d\n",
	    topology->iph[i],
	    topology->jph[i],
	    topology->kph[i],
	    topology->lph[i],
	    1,                               /* 1 for dihedrals  */
	    topology->icph[i]);
  }

  
  fprintf(ofile,"\n");

} /* write_dihedrals */


void write_impropers(FILE *ofile,struct g96topology *topology) {

  int i;

  fprintf(ofile,"[ dihedrals ]\n");
  fprintf(ofile,";   ai     aj     ak     al  funct            c0            c1            c2            c3            c4            c5\n");

  /* Write out impropers not involving hydrogens */
  for (i=0;i<topology->nqhi;++i) {
    fprintf(ofile,"%6d %6d %6d %6d %6d  GROMOS96i_%d\n",
	    topology->iq[i],
	    topology->jq[i],
	    topology->kq[i],
	    topology->lq[i],
	    2,                               /* 2 for impropers  */
	    topology->icq[i]);
  }

  /* Write out impropers involving hydrogens */
  for (i=0;i<topology->nqhih;++i) {
    fprintf(ofile,"%6d %6d %6d %6d %6d  GROMOS96i_%d\n",
	    topology->iqh[i],
	    topology->jqh[i],
	    topology->kqh[i],
	    topology->lqh[i],
	    2,                               /* 2 for impropers  */
	    topology->icqh[i]);
  }

  
  fprintf(ofile,"\n");

} /* write_impropers */
